USER_AGENT_BAD_CHARS = '\"\''
USER_AGENT_BLACKLIST = ('coc_coc_browser', 'QQBrowser', 'WeChat', 'YaBrowser', 'zh-CN', 'zh_CN', 'zhihu')
USER_AGENT_LEN_RANGE = (40, 160)
DEVICE_CATEGORY_LEN_RANGE = (4, 16)
PLATFORM_LEN_RANGE = (4, 16)

MAX_PROFILES = 150
