# Generated by Django 3.2.13 on 2022-05-06 22:42

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('djspoofer', '0002_alter_proxy_url'),
    ]

    operations = [
        migrations.AlterField(
            model_name='fingerprint',
            name='_h2_fingerprint',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='fingerprints', to='djspoofer.h2fingerprint'),
        ),
        migrations.AlterField(
            model_name='fingerprint',
            name='_tls_fingerprint',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='fingerprints', to='djspoofer.tlsfingerprint'),
        ),
        migrations.AlterField(
            model_name='fingerprint',
            name='device_fingerprint',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='fingerprints', to='djspoofer.devicefingerprint'),
        ),
        migrations.AlterField(
            model_name='fingerprint',
            name='geolocation',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='fingerprints', to='djspoofer.geolocation'),
        ),
    ]
