# Generated by Django 3.2.13 on 2022-05-06 20:02

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('djspoofer', '0001_initial'),
    ]

    operations = [
        migrations.AlterField(
            model_name='proxy',
            name='url',
            field=models.TextField(),
        ),
    ]
